// defs.h - I/O Device Definitions
//
// Written by
//  Timothy Stark <sword7@speakeasy.org>
//
// This file is part of the TS10 Emulator.
// See ReadMe for copyright notice.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// I/O Device Mapping Table for Unibus/Q22-Bus Interface

typedef struct ioEntry MAP_IO;
struct ioEntry {
	MAP_IO *Next;      // Linked List
	char   *devName;   // Device Name
	char   *Descrip;   // Description
	void   *Device;    // I/O Device

	// Device Settings
	uint32 csrAddr;    // Base address
	uint32 nRegs;      // Number of Registers
	uint32 intLevel;   // Interrupt Level (IPL)
	uint32 intMask;    // Interrupt Level Mask
	uint32 intVector;  // Interrupt Vector

	// Interrupt/Vector Functions
	void (*SetVector)(MAP_IO *, int);
	void (*SendInterrupt)(MAP_IO *);

	// Read/Write Access Functions
	int (*ReadIO)(void *, uint32, uint16 *, uint32);
	int (*WriteIO)(void *, uint32, uint16, uint32);
};

typedef struct ioCallback UBA_CALL;
struct ioCallback {
	// Read/Write Data I/O Functions
	uint32 (*ReadData)(void *, uint32, uint32);
	void   (*WriteData)(void *, uint32, uint32, uint32);

	// Read/Write Block I/O Functions
	uint32 (*ReadBlock)(void *, uint32, uint8 *, uint32);
	uint32 (*WriteBlock)(void *, uint32, uint8 *, uint32);
	void   *(*GetHostAddr)(void *, uint32);

	// QBA/UBA Interface Routines
	int    (*SetMap)(void *, MAP_IO *); // SetMap Routine
};

#define UBA_OK       0  // Successful - Ok
#define UBA_ERROR    1  // Error
#define UBA_NXM      2  // Non-Existant Device/Memory
#define UBA_RESERVED 3  // Reserved Area

#define QBA_CALL     UBA_CALL
#define QBA_OK       UBA_OK
#define QBA_NXM      UBA_NXM
#define QBA_RESERVED UBA_RESERVED

#define UQ_CALL     UBA_CALL
#define UQ_OK       UBA_OK
#define UQ_ERROR    UBA_ERROR
#define UQ_NXM      UBA_NXM
#define UQ_RESERVED UBA_RESERVED

extern DEVINFO *uq_Devices[];
