// dte20.h - DTE-20 Ten-Elenve Interface Definitions
//
// Written by
//  Timothy Stark <sword7@speakeasy.org>
//
// This file is part of the TS10 Emulator.
// See ReadMe for copyright notice.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// (CONxx) Condition Bits on Device DTE.
#define DTE_RM     0100000 // Restricted Mode 11
#define DTE_DEAD11 0040000 // Power Failure on PDP-11
#define DTE_TO11DB 0020000 // Ring Doorbell to PDP-11
#define DTE_CR11B  0010000 // Clear Reload PDP-11 Button
#define DTE_SR11B  0004000 // Set Reload PDP-11 Button
#define DTE_TO10DB 0001000 // Ring Doorbell to PDP-10
#define DTE_CL11PT 0001000 // Clear Doorbell for PDP-10
#define DTE_TO11ER 0000400 // To-11 Error
#define DTE_TO11DN 0000100 // To-11 Normal Termination Flag (Done)
#define DTE_CLTO11 0000100 // Clear To-11 Normal Termination Flag
#define DTE_TO10DN 0000040 // To-10 Normal Termination Flag (Done)
#define DTE_CLTO10 0000040 // Clear To-10 Normal Termination Flag
#define DTE_PILDEN 0000020 // Enable Load PIA
#define DTE_TO10ER 0000020 // To-10 Error
#define DTE_PIOENB 0000010 // PIO Enable
#define DTE_PIA    0000007 // PIA

#define DTE_CLR    (DTE_CL11PT|DTE_CLTO11|DTE_CLTO10)

// (DATAx) Data Bits on Device DTE.
#define TO10IB 0010000 // Interrupt after xfer
#define TO10BC 0007777 // To-10 Byte Count Field

// DTE20 Hardware EPT Locations
#define DTEHBG 0140 // Beginning of DTE20 Hardware EPT Location
#define DTEEBP 0140 // To-11 Byte Pointer
#define DTETBP 0141 // To-10 Byte Pointer
#define DTEDII 0142 // Interrupt Location
//             0143 // Unused
#define DTEEPW 0144 // Examine Protection Word
#define DTEERW 0145 // Examine Relocation Word
#define DTEDPW 0146 // Deposit Protection Word
#define DTEDRW 0147 // Deposit Relocation Word
#define DTEHBE 0147 // End of DTE20 Hardware EPT Location

// *************************************
// ****** Secondary Protocol Area ******
// *************************************

// DTE20 Secondary Protocol Locations
//   11 = PDP10 Rings PDP11 Doorbell
//   10 = PDP11 Rings PDP10 Doorbell

#define DTEFLG 0444 //      Operation Complete Flag
//             0445 //      |
//             0446 //      | Unused
//             0447 //      |
#define DTEF11 0450 //      Chracter from CTY to PDP-10
#define DTECMD 0451 // (11) PDP-10 to PDP-11 Command Word
//             0452 //      |
//             0453 //      | Unused
//             0454 //      |
#define DTEMTD 0455 // (10) CTY Output Complete Flag
#define DTEMTI 0456 // (10) CTY Input Flag

// DTE20 Secondary Protocol Command Field
#define CMD_MTO 010 // CTY Output
#define CMD_ESP 011 // Monitor Mode On  (Enter Secondary Protocol)
#define CMD_LSP 012 // Monitor Mode Off (Leave Secondary Protocol)
#define CMD_GDT 013 // Get Date/Time

// DTECMD Command Word
#define CMD_M_CODE 017  // Mask of Command Code
#define CMD_P_CODE 8    // Position of Command Code
#define MTO_CHAR   0377 // MTO: ASCII Character
#define LSP_RCOM   01   // LSP: Reset Communication Region Area

// DTEMTD (Output Done Flag)
#define MTD_ODN 0x8000 // Terminal Output Done
#define MTD_CSH 0x0002 // Cache Enable by KLI
#define MTD_GDT 0x0001 // Zero if GDT is supported

// DTE20 Time of Day format for Secondary Protocol
// 
//     0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 2 2 2 2 2 2 2 2 2 2 3 3
//     0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
//    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
// 0: |       Valid (Always -1)       |              Year             |
//    +-+-+-+-^-+-+-+-^-+-+-+-^-+-+-+-^-+-+-+-^-+-+-+-^-+-+-+-^-+-+-+-+
// 1: |     Month     |      Day      |  Day of Week  |D|  Time Zone  |
//    +-+-+-+-^-+-+-+-^-+-+-+-^-+-+-+-^-+-+-+-^-+-+-+-^-+-+-+-^-+-+-+-+
// 2: |     Seconds Since Midnight    |0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0|
//    +-+-+-+-^-+-+-+-^-+-+-+-^-+-+-+-^-+-+-+-^-+-+-+-^-+-+-+-^-+-+-+-+
//     0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 2 2 2 2 2 2 2 2 2 2 3 3 3 3 3 3 
//     4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5

#define TOD_VALID 0xFFFF0000 // 0 Valid - Always -1
#define TOD_YEAR  0x0000FFFF // 0 Year
#define TOD_MONTH 0xFF000000 // 1 Month
#define TOD_DAY   0x00FF0000 // 1 Day of Month
#define TOD_WDAY  0x0000FF00 // 1 Day of Week
#define TOD_DST   0x00000080 // 1 Day Saving Time Flag
#define TOD_TZ    0x0000007F // 1 Time Zone
#define TOD_SECS  0xFFFF0000 // 2 Seconds since midnight / 2

// *************************************
// ******* Primary Protocol Area *******
// *************************************

// Communication Region Area for each Processor

#define nPIDENT   000 // Processor Identification Word
#define nCNTPNT   001 // Pointer to Communication Area of Next Processor
#define nTOD      002 // Time of Day
#define nDATE     003 // Date
#define nKALIVE   004 // Keep Alive Counter
#define nEPTADR   015 // Address of Executive Process Table (EPT)
#define nLDADR    016 // Start Address of 10-Loader
#define nMONADR   017 // Start Address of 10-Monitor
#define nFORPRO   020 // For Processor Identification Word
#define nPROPRNT  021 // 
#define nSTATUS   022 // Status Word
#define nQSIZE    023 // Queue Size Word
#define nCTY0CW   024 // CTY0 Command Word
#define nCTY0RW   025 // CTY0 Response Word
#define nMISCW    030 // Misc Command Word
#define nMISRW    031 // Misc Response Word
#define nDATCW    032 // Data Command Word
#define nDATRW    033 // Data Response Word


// Word 0  PIDENT - Processor Identification Word
//
// |<---------- Left Halfword -------->|<--------- Right Halfword -------->|
// +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
// |T|CVER |    PVER     |  NPRO     |CASZ |           PNAM                |
// +-+-+-^-+-+-^-+-+-^-+-+-^-+-+-^-+-+-^-+-+-^-+-+-^-+-+-^-+-+-^-+-+-^-+-+-+
//  0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 2 2 2 2 2 2 2 2 2 2 3 3 3 3 3 3
//  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5
//
// Label  Description
// -----  -----------
// T      Ten Indicator
// CVER   Communication Version
// PVER   Protocol Version
// NPRO   Number of Processors
// CASZ   Communication Area Size (In Multiplies of 8)
// PNAM   Procesor Name

#define PID_TIND 0400000000000 // Ten Processor Indicator
#define PID_CVER 0340000000000 // Communication Version
#define PID_PVER 0037600000000 // Protocol Version
#define PID_NPRO 0000176000000 // Number of Processors
#define PID_CASZ 0000001600000 // Block Size (In multiplies of 8)
#define PID_PNAM 0000000177777 // Processor Name

#define PID_M_CVER 03
#define PID_M_PVER 0177
#define PID_M_NPRO 077
#define PID_M_CASZ 07
#define PID_M_PNAM 0177777

// Word 22  STATUS - Status Word
//
// |<---------- Left Halfword -------->|<--------- Right Halfword -------->|
// +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
// |P|L|I|V|F|N|0 0 0 0 0 0 0|Q|0 0 0|W|I|X|    TO10IC     |    TO11IC     |
// +-+-+-^-+-+-^-+-+-^-+-+-^-+-+-^-+-+-^-+-+-^-+-+-^-+-+-^-+-+-^-+-+-^-+-+-+
//  0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 2 2 2 2 2 2 2 2 2 2 3 3 3 3 3 3
//  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5
//
// Label    Description
// -----    -----------
// P        Power Fail Indicator
// L        Load -11 Code Indicator
// I        Initialize (QP2)
// V        Valid Examine Test
// F        CTY In Use Indicator
// N        CTY In Use Number
// Q        Queue'd In Use
// W        16-Bit Mode
// I        Indirect Data
// X        Processor is transferring queue
// TO10IC   TO10 Interrupt Count
// TO11IC   TO11 Interrupt Count

#define ST_POWERF  0400000000000 // Power Fail Indicator
#define ST_LOAD11  0200000000000 // Load -11 Code Indicator
#define ST_INIT    0100000000000 // Initialize (QP2)
#define ST_VALID   0040000000000 // Valid Examine (Must be one)
#define ST_CTYIUF  0020000000000 // CTY In Use Indicator
#define ST_CTYIUN  0010000000000 // CTY In Use Number
#define ST_QPIU    0000020000000 // Queue'd Protocol In Use
#define ST_16BT    0000001000000 // Transfer: 16-Bit Mode
#define ST_IND     0000000400000 // Transfer: Indirect Data
#define ST_TOIT    0000000200000 // Processor is transferring queue
#define ST_RCV     0000000200000 // Received (QP2)
#define ST_TO10IC  0000000177600 // TO-10 Interrupt Count
#define ST_TO11IC  0000000000377 // TO-11 Interrupt Count

#define ST_P_TO10IC 8
#define ST_M_IC     0377

// TO10Q/TO11Q Message Field - Function Codes
#define FNC_2EI 001 // TO-11 Initial Message
#define FNC_2TI 002 // TO-10 Initial Message
#define FNC_STR 003 // String Data
#define FNC_LNC 004 // Line-Char, Line-Char
#define FNC_RDS 005 // Request Device Status
#define FNC_HDS 007 // Here is Device Status
#define FNC_RDT 011 // Request Date and Time
#define FNC_HDT 012 // Here is Date and Time
#define FNC_FLO 013 // Flush Output (^O)
#define FNC_SNA 014 // Send All (All terminal lines)
#define FNC_DSC 015 // Dataset Connected
#define FNC_HUD 016 // Hang Up Dataset
#define FNC_ACK 017 // Acknowledge (Device Line Allocation Free)
#define FNC_XOF 020 // XOFF TTY Line
#define FNC_XON 021 // XON TTY Line
#define FNC_HLS 022 // Here are line speeds for TTYs
#define FNC_HLA 023 // Here are allocations
#define FNC_RBI 024 // TO-10 Reboot Information
#define FNC_AKA 025 // Acknowledge All
#define FNC_TDO 026 // Turn Device (On/Off)
#define FNC_EDR 027 // Enable/Disable Remotes
#define FNC_LDR 030 // Load LP RAM
#define FNC_LDV 031 // Load LP VFU
#define FNC_D6D 032 // DAS60 Data
#define FNC_KPS 033 // KLINIK Parameter Status
#define FNC_AXF 034 // Enable/Disable Auto-XOFF
#define FNC_BTC 035 // TO-11 Break Through Character Data
#define FNC_DBN 036 // TO-11 Turn On Debug Mode (KL Mode)
#define FNC_DBF 037 // TO-11 Turn Off Debug Mode

// Here is Device Status Defintions

#define HDS_CFG 0004000 // Configuration Information
#define HDS_LIN 0002000 // Lost Interrupt Enable
#define HDS_HNG 0001000 // Hung Device
#define HDS_F11 0000400 // FILES-11
#define HDS_FER 0000200 // Fatal Error
#define HDS_ELR 0000100 // Error Logging Request
#define HDS_EOF 0000040 // End of File
#define HDS_IOP 0000020 // I/O In Progress
#define HDS_SER 0000010 // Soft Error
#define HDS_HER 0000004 // Hard Error
#define HDS_OFL 0000002 // Off-Line Flag
#define HDS_NXD 0000001 // Non-Existant Device

#define PRIMARY   0  // Primary Protocol
#define SECONDARY 1  // Secondary Protocol

#define CTY_PORT   5000 // Default CTY port
#define CTY_ESCAPE 0x1C // Default ASCII code

typedef struct tblDevice DTE20_DEVICE;

struct tblDevice {
	int32  idChannel;    // Channel Identification
	int32  idDevice;     // Device Identification
	int32  Protocol;     // Protocol Flag (Primary = 0, Secondary = 1)
	uint16 srFlags;      // Status Register
	uint16 mtdFlags;     // Done Register
	int32  t10bCount;    // TO10 - Byte Count/Interrupt after xfer

	// TO10/TO11 Message Field
	uint16 t11Length;      // TO11 Message: Length
	uint8  t11Data[256];   // TO11 Message: Data
	uint16 t10Length;      // TO10 Message: Length
	uint8  t10Data[256];   // TO10 Message: Data

	CLK_QUEUE *Timer;

	SOCKET *ctyServer;   // CTY Listening Socket
	SOCKET *ctySocket;   // CTY Socket

	// Console TTY Buffer
	uchar inBuffer[4096];
	char  outBuffer[4096];
	int   idxInQueue, idxOutQueue;
	int   idxOutBuffer;
	char  lastSeen;
};
