// ks10_sys.c - KS10 System Configuration
//
// Copyright (c) 2001, Timothy M. Stark
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// TIMOTHY M STARK BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
// CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of Timothy M Stark shall not
// be used in advertising or otherwise to promote the sale, use or other 
// dealings in this Software without prior written authorization from
// Timothy M Stark.

#include "pdp10/defs.h"
#include "pdp10/ks10.h"

#define KS10_KEY     "KS10"
#define KS10_NAME    "KS10 Emulator"
#define KS10_VERSION "v0.8.9 (Late Alpha)"

typedef struct ks10_Device KS10_DEVICE;
struct ks10_Device {
	P10_CPU *Processor;
};

// Look at complete information at bottom of this file.
DEVINFO KS10_Processor;

void *ks10_Create(MAP_DEVICE *nmap, MAP_DEVICE *map, int argc, char **argv)
{
	KS10_DEVICE *ks10;
	DEVINFO     *iptr, *cptr;
	int         idx;

	if (nmap == NULL) {
		if (ks10 = (KS10_DEVICE *)calloc(1, sizeof(KS10_DEVICE))) {
			map->dtName     = KS10_Processor.dtName;
			map->emuName    = KS10_Processor.emuName;
			map->emuVersion = KS10_Processor.emuVersion;
			map->devInfo    = &KS10_Processor;
			map->Device     = ks10;

			printf("Created %s on %s: - %s %s\n",
				map->dtName, map->devName, map->emuName, map->emuVersion);
		}
		return ks10;
	}
}

int ks10_Reset(void *dptr)
{
	KS10_DEVICE *ks10 = (KS10_DEVICE *)dptr;
 	P10_CPU     *p10  = ks10->Processor;
	int         ac;

	// Clear all ammulators
	for (ac = 0; ac < p10->nAccumlators; ac++)
		p10->acBlocks[ac] = 0;

	return EMU_OK;
}

int ks10_Setup(void *dptr, P10_CPU *p10)
{
	KS10_DEVICE *ks10 = (KS10_DEVICE *)dptr;

	// Set KS10 configuration flag
	p10->cnfFlags     = CNF_KS10;
	p10->szBlock      = ACBLKSIZE;
	p10->nBlocks      = NACBLOCKS;
	p10->nAccumlators = p10->nBlocks * p10->szBlock;
	p10->acBlocks =
		(int36 *)calloc(p10->nAccumlators, sizeof(int36));

	// Processor Initialization
	ks10->Processor = p10;
	ks10_Reset(ks10);

	return EMU_OK;
}

DEVINFO KS10_Devices[] =
{
	NULL
};

DEVINFO KS10_Processor =
{
	KS10_KEY,          // Key Name (Device Type)
	KS10_NAME,         // Emulator Name
	KS10_VERSION,      // Emulator Version
	KS10_Devices,      // Lisiting of KS10 Devices
	DF_USE,            // Device Flags
	DT_PROCESSOR,      // Device Type

	NULL, NULL, NULL,

	ks10_Create,       // Create Routine
	NULL,              // Delete Routine
	ks10_Setup,        // Setup Routine
	ks10_Reset,        // Reset Routine
	NULL,              // Attach Routine
	NULL,              // Detach Routine
	NULL,              // Info Routine
	NULL,              // Execute Routine
};

