// cpu.c - PDP11 Emulator (Main Instructions)
//
// Written by
//  Timothy Stark <sword7@speakeasy.org>
//
// This file is part of the TS10 Emulator.
// See ReadMe for copyright notice.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <pdp11/defs.h>


char *p11_DisplayConditions(void)
{
	static   char *ccStr[5];
	register uint16 sr = SR;

	// Display condition bits
	ccStr[0] = (cc & CC_N) ? 'N' : '-';
	ccStr[1] = (cc & CC_Z) ? 'Z' : '-';
	ccStr[2] = (cc & CC_V) ? 'V' : '-';
	ccStr[3] = (cc & CC_C) ? 'C' : '-';
	ccStr[4] = '\0';
}

// Condition Instructions
// NOP, CCC, CLC, CLN, CLV, CLZ, SCC, SEC, SEN, SEV, and SEZ
// Opcode 000240-000257

void p11_Opcode_CC(void)
{
#ifdef DEBUG
	uint16 osr = SR;
#endif /* DEBUG */

	if (opCode & CC_SET)
		SR |= (opCode & CC_ALL)
	else
		SR &= ~(opCode & CC_ALL);

#ifdef DEBUG
	if (dbg_Check(DBG_TRACE|DBG_DATA)) {
		dbg_Printf("CC: %s => %s\n",
			p11_DisplayConditions(osr), p11_DisplayConditions(SR));
	}
#endif /* DEBUG */
}
