// defs.h - PDP-11 Series Configuration
//
// Written by
//  Timothy Stark <sword7@speakeasy.org>
//
// This file is part of the TS10 Emulator.
// See ReadMe for copyright notice.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

typedef struct tblCPUDevice PDP11_CPU;
struct tblCPUDevice {
	int16  gpRegs[8]; // 8 General Registers (R0-R7, SP, and PC)
	uint16 srFlags;   // Processor Status Register
	uint16 mrFlags;   // Mode Register
};

// Processor Status Register
//
// +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
// |0  0  0  0  0  0  0  0 |  Prio  |T |N |Z |V |C |
// +--^--+--+--^--+--+--^--+--+--^--+--+--^--+--+--+
//  15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00
//
// Prio = Priority
// T    = Trace/Trap
// N    = Negative
// Z    = Zero
// V    = Overflow
// C    = Carry

#define FLG_PRIORITY 0000340 // Priority
#define FLG_TRACE    0000020 // Trace Flag
#define FLG_TRAP     0000020 // Trap Flag
#define FLG_NEGATIVE 0000010 // Negative Flag
#define FLG_ZERO     0000004 // Zero Flag
#define FLG_OVERFLOW 0000002 // Overflow Flag
#define FLG_CARRY    0000001 // Carry Flag

#define FLG_T  FLG_TRAP     // Trace/Trap Flag
#define FLG_N  FLG_NEGATIVE // Negative Flag
#define FLG_Z  FLG_ZERO     // Negative Flag
#define FLG_V  FLG_OVERFLOW // Overflow Flag
#define FLG_C  FLG_CARRY    // Carry Flag

// Condition Code Flags
#define CC_SET 0000020      // Set bit in CC instructions
#define CC_N   FLG_NEGATIVE // Negative Flag
#define CC_Z   FLG_ZERO     // Negative Flag
#define CC_V   FLG_OVERFLOW // Overflow Flag
#define CC_C   FLG_CARRY    // Carry Flag
#define CC_ALL (CC_N|CC_Z|CC_V|CC_C)

// Mode Register
//
// +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
// |  R/S   |TU|WM|MS|DS|ND|                 |LS|CP|
// +--^--+--+--^--+--+--^--+--+--^--+--+--^--+--+--+
//  15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00
//
// Label      Description
// -----      -----------
// R/S        Start/Restart
// TU         Tester/User Mode
// WM         16-Bit/8-Bit Bus
// MS         64K/4K or 16K Memory
// DS         Dynamic/Static Memory
// ND         Normal/Delay Read/Write
// LS         Long/Standard Microcycle
// CP         Constant/Processor Mode Clock

// General Register Defintions

#define GPREG(rn) pdp11->gpRegs[rn]

#define R0 GPREG(0)  // R0 General Register
#define R1 GPREG(1)  // R1 General Register
#define R2 GPREG(2)  // R2 General Register
#define R3 GPREG(3)  // R3 General Register
#define R4 GPREG(4)  // R4 General Register
#define R5 GPREG(5)  // R5 General Register
#define R6 GPREG(6)  // R6 General Register
#define R7 GPREG(7)  // R7 General Register
#define SP GPREG(6)  // Stack Pointer Register   (R6)
#define PC GPREG(7)  // Program Counter Register (R7)

#define SR pdp11->srFlags // Status Register
#define MR pdp11->mrFlags // Mode Register
