// memory.c - PDP-11 Memory System
//
// Written by
//  Timothy Stark <sword7@speakeasy.org>
//
// This file is part of the TS10 Emulator.
// See ReadMe for copyright notice.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

typedef struct tblMemory PDP11_MEM;
struct tblMemory {
	int16 *Memory;
	int32 Size;
};


void *p11_CreateMemory(int32 Size)
{
	PDP11_MEM *mem11;

	if (mem11 = (PDP11_MEM *)calloc(1, sizeof(PDP11_MEM))) {
		mem11->Memory = (int16 *)calloc(1, Size);
		mem11->Size   = Size;
	}

	return (void *)mem11;
}

void p11_FreeMemory(PDP11_MEM *mem11)
{
	if (mem11) {
		if (mem11->Memory)
			free(mem11->Memory);
		free(mem11);
	}
}

inline int16 p11_pRead(PDP11_MEM *mem11, int32 pAddr)
{
	if (pAddr < mem11->Size) {
		return mem11->Memory[pAddr >> 1];
	}
	return 0;
}

inline void p11_pWrite(PDP11_MEM *mem11, int32 pAddr, int16 Data)
{
	if (pAddr < mem11->Size)
		mem11->Memory[pAddr >> 1] = Data;
}
