// cpu_inst.c - TMS9900 Series Instruction Functions
//
// Copyright (c) 2001, Timothy M. Stark
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// TIMOTHY M STARK BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
// CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of Timothy M Stark shall not
// be used in advertising or otherwise to promote the sale, use or other 
// dealings in this Software without prior written authorization from
// Timothy M Stark.

#include "ti99/defs.h"

// A    Add
INSFUNC(ti99, A)
{
	register int16 src, dst;

	dst += src;
}

// AB   Add Byte
INSFUNC(ti99, AB)
{
}

// ABS  Aboslute Value
INSFUNC(ti99, ABS)
{
}

// AI   Add Immediate
INSFUNC(ti99, AI)
{
}

// ANDI AND Immediate
INSFUNC(ti99, ANDI)
{
}

// B    Branch
INSFUNC(ti99, B)
{
}

// BL   Branch and Link
INSFUNC(ti99, BL)
{
}

// BLWP Branch and Load Workplace Pointer
INSFUNC(ti99, BLWP)
{
}

// C    Compare
INSFUNC(ti99, C)
{
	register int16 src, dst;

	CC_IIIPP_W(src, dst);
}

// CB   Compare Byte
INSFUNC(ti99, CB)
{
	register int16 src, dst;

	CC_IIIPP_B(src, dst);
}

// CI   Compare Immediate
INSFUNC(ti99, CI)
{
}

// CKOF Clock Off
INSFUNC(ti99, CKOF)
{
}

// CKON Clock On
INSFUNC(ti99, CKON)
{
}

// CLR  Clear
INSFUNC(ti99, CLR)
{
}

// COC  Compare Ones Corresponding
INSFUNC(ti99, COC)
{
}

// CZC  Compare Ones Corresponding
INSFUNC(ti99, CZC)
{
}

// DEC  Decrement
INSFUNC(ti99, DEC)
{
}

// DECT Decrement by Two
INSFUNC(ti99, DECT)
{
}

// DIV  Divide
INSFUNC(ti99, DIV)
{
}

// IDLE Computer Idle
INSFUNC(ti99, IDLE)
{
}

// INC  Increment
INSFUNC(ti99, INC)
{
}

// INCT Increment by Two
INSFUNC(ti99, INCT)
{
}

// INV  Invert
INSFUNC(ti99, INV)
{
}

// JEQ  Jump if Equal
INSFUNC(ti99, JEQ)
{
}

// JGT  Jump if Greater Than
INSFUNC(ti99, JGT)
{
}

// JH   Jump if High
INSFUNC(ti99, JH)
{
}

// JHE  Jump if High or Equal
INSFUNC(ti99, JHE)
{
}

// JL   Jump if Low
INSFUNC(ti99, JL)
{
}

// JLE  Jump if Low or Equal
INSFUNC(ti99, JLE)
{
}

// JLT  Jump if Less Than
INSFUNC(ti99, JLT)
{
}

// JMP  Jump unconditionally
INSFUNC(ti99, JMP)
{
}

// JNC  Jump if No Carry
INSFUNC(ti99, JNC)
{
}

// JNE  Jump if Not Equal
INSFUNC(ti99, JNE)
{
}

// JNO  Jump if No Overflow
INSFUNC(ti99, JNO)
{
}

// JOC  Jump On Carry
INSFUNC(ti99, JOC)
{
}

// JOP  Jump if Odd Parity
INSFUNC(ti99, JOP)
{
}

// LDCR Load Communication Register
INSFUNC(ti99, LDCR)
{
}

// LI   Load Immediate
INSFUNC(ti99, LI)
{
}

// LIMI Load Interrupt Mask Immediate
INSFUNC(ti99, LIMI)
{
}

// LREX Load or Restart Execution
INSFUNC(ti99, LREX)
{
}

// LWPI Load Workplace Pointer Immediate
INSFUNC(ti99, LWPI)
{
}

// MOV  Move
INSFUNC(ti99, MOV)
{
	register int16 src, dst;

	CC_IIIPP_W(dst, 0);
}

// MOVB Move Byte
INSFUNC(ti99, MOVB)
{
	register int16 src, dst;

	CC_IIIPP_B(dst, 0);
}

// MPY  Multiply
INSFUNC(ti99, MPY)
{
}

// NEG  Negate
INSFUNC(ti99, NEG)
{
}

// ORI  OR Immediate
INSFUNC(ti99, ORI)
{
}

// RSET Reset
INSFUNC(ti99, RSET)
{
}

// RTWP Return Workplace Pointer
INSFUNC(ti99, RTWP)
{
}

// S    Subtract
INSFUNC(ti99, S)
{
}

// SB   Subtract Byte
INSFUNC(ti99, SB)
{
}

// SBO  Set Bit to One
INSFUNC(ti99, SBO)
{
}

// SBZ  Set Bit to Zero
INSFUNC(ti99, SBZ)
{
}

// SETO Set to Ones
INSFUNC(ti99, SETO)
{
}

// SLA  Shift Left Arithmetic
INSFUNC(ti99, SLA)
{
	register uint16 src, dst, cnt;

	// Shift Left
	dst = src << cnt;
	CC_IIIPP_W((int16)dst, 0);

#ifdef DEBUG
	if (dbg_Check(DBG_TRACE|DBG_DATA))
		dbg_Printf("SLA: %04X << %d => %04X\n", src, cnt, dst);
#endif /* DEBUG */
}

// SOC  Set Ones Corresponding
INSFUNC(ti99, SOC)
{
}

// SOCB Set Ones Corresponding Byte
INSFUNC(ti99, SOCB) 
{
}

// SRA  Shift Right Arithmetic
INSFUNC(ti99, SRA)
{
	register int16  src, dst;
	register uint16 cnt;

	// Shift Right Arithmetic
	dst = src >> cnt;
	CC_IIIPP_W(dst, 0);

#ifdef DEBUG
	if (dbg_Check(DBG_TRACE|DBG_DATA))
		dbg_Printf("SRA: %04X >> %d => %04X\n", src, cnt, dst);
#endif /* DEBUG */
}

// SRC  Shift Right Circular
INSFUNC(ti99, SRC)
{
	register uint16 src, dst, cnt;

	// Rotate Right
	dst = (src >> cnt) | (src << cnt);
	CC_IIIPP_W((int16)dst, 0);

#ifdef DEBUG
	if (dbg_Check(DBG_TRACE|DBG_DATA))
		dbg_Printf("SRC: %04X >> %d => %04X\n", src, cnt, dst);
#endif /* DEBUG */
}

// SRL  Shift Right Logical
INSFUNC(ti99, SRL)
{
	register uint16 src, dst, cnt;

	// Shift Right Logical
	dst = src >> cnt;
	CC_IIIPP_W((int16)dst, 0);

#ifdef DEBUG
	if (dbg_Check(DBG_TRACE|DBG_DATA))
		dbg_Printf("SRA: %04X >> %d => %04X\n", src, cnt, dst);
#endif /* DEBUG */
}

// STCR Store Communication Register
INSFUNC(ti99, STCR)
{
}

// STST Store Status Register
INSFUNC(ti99, STST)
{
}

// STWP Store Workplace Pointer
INSFUNC(ti99, STWP)
{
}

// SWPB Swap Bytes
INSFUNC(ti99, SWPB)
{
	register uint16 src, dst;

	// Swap two bytes
	dst = (src >> 8) | (src << 8);

#ifdef DEBUG
	if (dbg_Check(DBG_TRACE|DBG_DATA))
		dbg_Printf("SWPB: %04X => %04X\n", src, dst);
#endif /* DEBUG */
}

// SZC  Set Zeros Corresponding
INSFUNC(ti99, SZC)
{
}

// SZCB Set Zeros Corresponding Byte
INSFUNC(ti99, SZCB)
{
}

// TB   Test Bit
INSFUNC(ti99, TB)
{
}

// X    Execute
INSFUNC(ti99, X)
{
}

// XOP  Extended Operation
INSFUNC(ti99, XOP)
{
}

// XOR  Exclusive OR
INSFUNC(ti99, XOR)
{
	register int16 src, dst, res;

	// Exclusive OR (XOR)
	res = src ^ dst;
	CC_IIIPP_W(res, 0);

#ifdef DEBUG
	if (dbg_Check(DBG_TRACE|DBG_DATA))
		dbg_Printf("XOR: %04X ^ %04X => %04X\n", src, dst, res);
#endif /* DEBUG */
}

// Illegal Instruction
INSFUNC(ti99, Illegal)
{
#ifdef DEBUG
//	dbg_Printf("%s: Illegal Instruction (%04X) at PC %04X\n",
//		ti99cpu->dtName, ti99cpu->opCode, faultPC);
#endif /* DEBUG */
}
