// defs.h - TI99 Emulator Definitions
//
// Copyright (c) 2001, Timothy M. Stark
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// TIMOTHY M STARK BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
// CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of Timothy M Stark shall not
// be used in advertising or otherwise to promote the sale, use or other 
// dealings in this Software without prior written authorization from
// Timothy M Stark.

#include "emu/defs.h"

typedef struct ti99_InstTable TI99_INST;
typedef struct ti99_Processor TI99_CPU;
typedef struct ti99_System    TI99_SYSTEM;

struct ti99_InstTable {
	char   *Name;      // Mnemonic Name
	char   *Descrip;   // Description
	uint16 opCode;     // Opcode
	uint8  opFormat;   // Opcode Format
	uint32 Flags;      // Flags

	void   (*Execute)(register TI99_CPU *);
};

struct ti99_Processor {
	// Description Information
	char   *dtName;     // Device Type Name
	char   *emuName;    // Emulator Name
	char   *emuVersion; // Emulator Version

	uint16 pc;      // Program Counter
	uint16 wp;      // Workplace Pointer
	uint16 st;      // Status Register
	uint16 cc;      // Condition Code (Part of Status Register)

	uint16 faultPC; // faulting PC pointer
	uint32 ips;     // Instructions Per Second
};

struct ti99_System {
	// Description Information
	char   *dtName;     // Device Type Name
	char   *emuName;    // Emulator Name
	char   *emuVersion; // Emulator Version

	TI99_CPU *Processor;
};

#define TI99 ti99cpu
#define PC      TI99->pc
#define WP      TI99->wp
#define ST      TI99->st
#define CC      TI99->cc

#define faultPC TI99->faultPC
#define IPS     TI99->ips

// Status Register
//
// +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
// |L>|A>|EQ|C |OV|OP|X |              | Interrupt |
// +--+--+--+--^--+--+--+--^--+--+--+--^--+--+--+--+
//  00 01 02 03 04 05 06 07 08 09 10 11 12 13 14 15

#define SR_L    0x8000    // Logical Greater Than
#define SR_A    0x4000    // Arithmetic Greater Than
#define SR_E    0x2000    // Equal Bit
#define SR_C    0x1000    // Carry Bit
#define SR_V    0x0800    // Overflow Bit
#define SR_P    0x0400    // Odd Parity Bit
#define SR_X    0x0200    // Extended Operation Bit
#define SR_I    0x000F    // Interrupt Mask

#define SR_CC   (SR_L|SR_A|SR_E|SR_C|SR_V|SR_P)

#define CC_LGT SR_L // Logical Greater Than
#define CC_AGT SR_A // Arithmetic Greater Than
#define CC_EQ  SR_E // Equal
#define CC_C   SR_C // Carry
#define CC_OV  SR_V // Overflow
#define CC_OP  SR_P // Odd Parity

// Instruction Function Call Definition
#define INSNAME(cpu, rtn) cpu##_op##rtn
#define INSFUNC(cpu, rtn) \
	void INSNAME(cpu, rtn)(register TI99_CPU *ti99cpu)

// Update Condition Codes

//#define CC_IIIZZ_B(r) \
//	if ((uint16)r > 0) CC = CC_LGT | ((r > 0) ? CC_AGT : 0); \
//	else               CC = CC_EQ;

//#define CC_IIIZZ_W(r) \
//	if ((uint16)r > 0) CC = CC_LGT | ((r > 0) ? CC_AGT : 0); \
//	else               CC = CC_EQ;

#define CC_IIIPP_B(r, c) \
	if (r < c)      CC = CC_LGT; \
	else if (r > c) CC = CC_LGT|CC_AGT; \
	else            CC = CC_EQ;

#define CC_IIIPP_W(r, c) \
	if (r < c)      CC = CC_LGT; \
	else if (r > c) CC = CC_LGT|CC_AGT; \
	else            CC = CC_EQ;

// External Defintions
#include "ti99/proto.h"
