// inst.c - TMS9900 Series and GPL Instruction Table
//
// Copyright (c) 2001, Timothy M. Stark
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// TIMOTHY M STARK BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
// CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of Timothy M Stark shall not
// be used in advertising or otherwise to promote the sale, use or other 
// dealings in this Software without prior written authorization from
// Timothy M Stark.
 
#include "ti99/defs.h"

// ********************************
// TMS9900 Series Assembly Language
// ********************************

#define A    "Add"
#define AB   "Add Bytes"
#define ABS  "Absolute Value"
#define AI   "Add Immediate"
#define ANDI "AND Immediate"
#define B    "Branch"
#define BL   "Branch and Link"
#define BLWP "Branch and Load Workplace Pointer"
#define C    "Compare"
#define CB   "Compare Bytes"
#define CI   "Compare Immediate"
#define CKOF "Clock Off"
#define CKON "Clock On"
#define CLR  "Clear"
#define COC  "Compare Ones Corresponding"
#define CZC  "Compare Zeros Corresponding"
#define DEC  "Decrement"
#define DECT "Decrement by Two"
#define DIV  "Divide"
#define IDLE "Computer Idle"
#define INC  "Increment"
#define INCT "Increment by Two"
#define INV  "Invert"
#define JEQ  "Jump if Equal"
#define JGT  "Jump if Greater Than"
#define JH   "Jump if High"
#define JHE  "Jump if High or Equal"
#define JL   "Jump if Low"
#define JLE  "Jump if Low or Equal"
#define JLT  "Jump if Less Than"
#define JMP  "Jump unconditionally"
#define JNC  "Jump if No Carry"
#define JNE  "Jump if Not Equal"
#define JNO  "Jump if No Overflow"
#define JOC  "Jump On Carry"
#define JOP  "Jump if Odd Parity"
#define LDCR "Load Communication Register"
#define LI   "Load Imemdiate"
#define LIMI "Load Interrupt Mask Immediate"
#define LREX "Load or Restart Execution"
#define LWPI "Load Workplace Pointer Immediate"
#define MOV  "Move"
#define MOVB "Move Bytes"
#define MPY  "Multiply"
#define NEG  "Negate"
#define NOP  "No Operation"
#define ORI  "OR Immediate"
#define RSET "Reset"
#define RT   "Return"
#define RTWP "Return Workplace Pointer"
#define S    "Subtract"
#define SB   "Subtract Bytes"
#define SBO  "Set Bit to One"
#define SBZ  "Set Bit to Zero"
#define SETO "Set to Ones"
#define SLA  "Shift Left Arithmetic"
#define SOC  "Set Ones Corresponding"
#define SOCB "Set Ones Corresponding Bytes"
#define SRA  "Shift Right Arithmetic"
#define SRC  "Shift Circular"
#define SRL  "Shift Right Logical"
#define STCR "Store Communication Register"
#define STST "Store Status Register"
#define STWP "Store Workplace Register"
#define SWPB "Swap Bytes"
#define SZC  "Set Zeros Corresponding"
#define SZCB "Set Zeros Corresponding Bytes"
#define TB   "Test Bit"
#define X    "Execute"
#define XOP  "Extended Operation"
#define XOR  "Exclusive OR"

// TMS9900
//TI99_FORMAT ti99_Format1      = { 1, 0xF000 };
//TI99_FORMAT ti99_Format2_JUMP = { 2, 0xFF00 };
//TI99_FORMAT ti99_Format2_CRU  = { 2, 0xFF00 };
//TI99_FORMAT ti99_Format3      = { 3, 0xFC00 };
//TI99_FORMAT ti99_Format4      = { 4, 0xFF00 };
//TI99_FORMAT ti99_Format5      = { 5, 0xFF00 };
//TI99_FORMAT ti99_Format6      = { 6, 0xFFC0 };
//TI99_FROMAT ti99_Format7      = { 7, 0xFFFF };
//TI99_FORMAT ti99_Format8      = { 8, 0xFFF0 };
//TI99_FORMAT ti99_Format9      = { 9, 0xFFC0 };

TI99_INST ti99_Inst[] =
{
	{ "A",    A,    0xA000, 1, 0, INSNAME(ti99, A)    },
	{ "AB",   AB,   0xB000, 1, 0, INSNAME(ti99, AB)   },
	{ "ABS",  ABS,  0x0740, 6, 0, INSNAME(ti99, ABS)  },
	{ "AI",   AI,   0x0220, 8, 0, INSNAME(ti99, AI)   },
	{ "ANDI", ANDI, 0x0240, 8, 0, INSNAME(ti99, ANDI) },
	{ "B",    B,    0x0440, 6, 0, INSNAME(ti99, B)    },
	{ "BL",   BL,   0x0680, 6, 0, INSNAME(ti99, BL)   },
	{ "BLWP", BLWP, 0x0400, 6, 0, INSNAME(ti99, BLWP) },
	{ "C",    C,    0x8000, 1, 0, INSNAME(ti99, C)    },
	{ "CB",   CB,   0x9000, 1, 0, INSNAME(ti99, CB)   },
	{ "CI",   CI,   0x0280, 8, 0, INSNAME(ti99, CI)   },
	{ "CKOF", CKOF, 0x03C0, 7, 0, INSNAME(ti99, CKOF) },
	{ "CKON", CKON, 0x03A0, 7, 0, INSNAME(ti99, CKON) },
	{ "CLR",  CLR,  0x04C0, 6, 0, INSNAME(ti99, CLR)  },
	{ "COC",  COC,  0x2000, 3, 0, INSNAME(ti99, COC)  },
	{ "CZC",  CZC,  0x2400, 3, 0, INSNAME(ti99, CZC)  },
	{ "DEC",  DEC,  0x0600, 6, 0, INSNAME(ti99, DEC)  },
	{ "DECT", DECT, 0x0640, 6, 0, INSNAME(ti99, DECT) },
	{ "DIV",  DIV,  0x3C00, 9, 0, INSNAME(ti99, DIV)  },
	{ "IDLE", IDLE, 0x0340, 7, 0, INSNAME(ti99, IDLE) },
	{ "INC",  INC,  0x0580, 6, 0, INSNAME(ti99, INC)  },
	{ "INCT", INCT, 0x05C0, 6, 0, INSNAME(ti99, INCT) },
	{ "INV",  INV,  0x0540, 6, 0, INSNAME(ti99, INV)  },
	{ "JEQ",  JEQ,  0x1300, 2, 0, INSNAME(ti99, JEQ)  },
	{ "JGT",  JGT,  0x1500, 2, 0, INSNAME(ti99, JGT)  },
	{ "JH",   JH,   0x1B00, 2, 0, INSNAME(ti99, JH)   },
	{ "JHE",  JHE,  0x1400, 2, 0, INSNAME(ti99, JHE)  },
	{ "JL",   JL,   0x1A00, 2, 0, INSNAME(ti99, JL)   },
	{ "JLE",  JLE,  0x1200, 2, 0, INSNAME(ti99, JLE)  },
	{ "JLT",  JLT,  0x1100, 2, 0, INSNAME(ti99, JLT)  },
	{ "JMP",  JMP,  0x1000, 2, 0, INSNAME(ti99, JMP)  },
	{ "JNC",  JNC,  0x1700, 2, 0, INSNAME(ti99, JNC)  },
	{ "JNE",  JNE,  0x1600, 2, 0, INSNAME(ti99, JNE)  },
	{ "JNO",  JNO,  0x1900, 2, 0, INSNAME(ti99, JNO)  },
	{ "JOC",  JOC,  0x1800, 2, 0, INSNAME(ti99, JOC)  },
	{ "JOP",  JOP,  0x1C00, 2, 0, INSNAME(ti99, JOP)  },
	{ "LDCR", LDCR, 0x3000, 4, 0, INSNAME(ti99, LDCR) },
	{ "LI",   LI,   0x0200, 8, 0, INSNAME(ti99, LI)   },
	{ "LIMI", LIMI, 0x0300, 8, 0, INSNAME(ti99, LIMI) },
	{ "LREX", LREX, 0x03E0, 7, 0, INSNAME(ti99, LREX) },
	{ "LWPI", LWPI, 0x02E0, 8, 0, INSNAME(ti99, LWPI) },
	{ "MOV",  MOV,  0xC000, 1, 0, INSNAME(ti99, MOV)  },
	{ "MOVB", MOVB, 0xD000, 1, 0, INSNAME(ti99, MOVB) },
	{ "MPY",  MPY,  0x3800, 9, 0, INSNAME(ti99, MPY)  },
	{ "NEG",  NEG,  0x0500, 6, 0, INSNAME(ti99, NEG)  },
	{ "NOP",  NOP,  0x1000, 7, 0, NULL                },
	{ "ORI",  ORI,  0x0260, 8, 0, INSNAME(ti99, ORI)  },
	{ "RSET", RSET, 0x0360, 7, 0, INSNAME(ti99, RSET) },
	{ "RT",   RT,   0x045B, 7, 0, NULL                },
	{ "RTWP", RTWP, 0x0380, 7, 0, INSNAME(ti99, RTWP) },
	{ "S",    S,    0x6000, 1, 0, INSNAME(ti99, S)    },
	{ "SB",   SB,   0x7000, 1, 0, INSNAME(ti99, SB)   },
	{ "SBO",  SBO,  0x1D00, 2, 0, INSNAME(ti99, SBO)  },
	{ "SBZ",  SBZ,  0x1E00, 2, 0, INSNAME(ti99, SBZ)  },
	{ "SETO", SETO, 0x0700, 6, 0, INSNAME(ti99, SETO) },
	{ "SLA",  SLA,  0x0A00, 5, 0, INSNAME(ti99, SLA)  },
	{ "SOC",  SOC,  0xE000, 1, 0, INSNAME(ti99, SOC)  },
	{ "SOCB", SOCB, 0xF000, 1, 0, INSNAME(ti99, SOCB) },
	{ "SRA",  SRA,  0x0800, 5, 0, INSNAME(ti99, SRA)  },
	{ "SRC",  SRC,  0x0B00, 5, 0, INSNAME(ti99, SRC)  },
	{ "SRL",  SRL,  0x0900, 5, 0, INSNAME(ti99, SRL)  },
	{ "STCR", STCR, 0x3400, 4, 0, INSNAME(ti99, STCR) },
	{ "STST", STST, 0x02C0, 8, 0, INSNAME(ti99, STST) },
	{ "STWP", STWP, 0x02A0, 8, 0, INSNAME(ti99, STWP) },
	{ "SWPB", SWPB, 0x06C0, 6, 0, INSNAME(ti99, SWPB) },
	{ "SZC",  SZC,  0x4000, 1, 0, INSNAME(ti99, SZC)  },
	{ "SZCB", SZCB, 0x5000, 1, 0, INSNAME(ti99, SZCB) },
	{ "TB",   TB,   0x1F00, 2, 0, INSNAME(ti99, TB)   },
	{ "X",    X,    0x0480, 6, 0, INSNAME(ti99, X)    },
	{ "XOP",  XOP,  0x2C00, 9, 0, INSNAME(ti99, XOP)  },
	{ "XOR",  XOR,  0x2800, 3, 0, INSNAME(ti99, XOR)  },
	{ NULL }
};

// *****************************
// Graphics Programming Language
// *****************************
