// system.c - TI99 Emulation System
//
// Copyright (c) 2001, Timothy M. Stark
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// TIMOTHY M STARK BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
// CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of Timothy M Stark shall not
// be used in advertising or otherwise to promote the sale, use or other 
// dealings in this Software without prior written authorization from
// Timothy M Stark.

#include "ti99/defs.h"

#define TI99_KEY     "TI99"
#define TI99_NAME    "TI99 Series Emulator"
#define TI99_VERSION "0.1 (Pre-Alpha)"




// ******************************************************

DEVINFO ti99_System; // Look at end of this file.

// Usage: create <device> <model> ...
void *ti99_Create(MAP_DEVICE *nmap, MAP_DEVICE *map, int argc, char **argv)
{
	TI99_SYSTEM *ti99sys;

	if (nmap == NULL) {
		// Set up new TI99 System
		if (ti99sys = (TI99_SYSTEM *)calloc(1, sizeof(TI99_SYSTEM))) {
			ti99sys->dtName     = TI99_KEY;
			ti99sys->emuName    = TI99_NAME;
			ti99sys->emuVersion = TI99_VERSION;

			map->dtName     = ti99sys->dtName;
			map->emuName    = ti99sys->emuName;
			map->emuVersion = ti99sys->emuVersion;
			map->devInfo    = &ti99_System;
			map->Device     = ti99sys;
			map->sysDevice  = ti99sys;
			map->sysMap     = map;
		}

		if (ti99sys)
			printf("Create %s on %s: - %s %s\n",
				map->dtName, map->devName, map->emuName, map->emuVersion);
		else
			printf("%s (%s): *** Can't Initialize System. Aborting. ***\n",
				map->devName, TI99_KEY);

		return ti99sys;
	} else {
		return NULL;
	}
}

extern DEVINFO ti99_System_994;
extern DEVINFO ti99_System_994A;

DEVINFO *ti99_Systems[] =
{
	&ti99_System_994,    // TI-99/4 Home Computer
	&ti99_System_994A,   // TI-99/4A Home Computer
//	&ti99_System_Geneve, // Geneve System
//	&ti99_System_99010,  // TI-990/10 Minicomputer
//	&ti99_System_99010A, // TI-990/10A Minicomputer
//	&ti99_System_99012,  // TI-990/12 Minicomputer
	NULL,
};

DEVINFO ti99_System =
{
	// Description Information
	TI99_KEY,     // Device Type Name
	TI99_NAME,    // Emulator Name
	TI99_VERSION, // Emulator Version
	ti99_Systems, // Listing of TI99 Series Systems
	DF_SELECT,    // Device Flags
	DT_SYSTEM,    // Device Type

	// Commands
	NULL, NULL, NULL,

	// Function Calls
	ti99_Create,    // Create Routine
	NULL,           // Delete Routine
	NULL,           // Setup Routine
	NULL,           // Reset Routine
	NULL,           // Attach Routine
	NULL,           // Detach Routine
	NULL,           // Info Routine
	NULL,           // Boot Routine
	NULL,           // Execute Routine
//	NULL,           // Debug Routine
};

