// proto.h - Prototypes for all .c files.
//
// Written by
//  Timothy Stark <sword7@speakeasy.org>
//
// This file is part of the TS-10 Emulator.
// See README for copyright notice.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// commands.c

// cpu_branch.c
int vax_SetBit(int32 *, int32);
int vax_TestBit(int32 *);
void vax_Opcode_ACBB(void);
void vax_Opcode_ACBW(void);
void vax_Opcode_ACBL(void);
void vax_Opcode_AOBLEQ(void);
void vax_Opcode_AOBLSS(void);
void vax_Opcode_BBC(void);
void vax_Opcode_BBS(void);
void vax_Opcode_BBCC(void);
void vax_Opcode_BBCS(void);
void vax_Opcode_BBSC(void);
void vax_Opcode_BBSS(void);
void vax_Opcode_BCC(void);
void vax_Opcode_BCS(void);
void vax_Opcode_BEQL(void);
void vax_Opcode_BGEQ(void);
void vax_Opcode_BGTR(void);
void vax_Opcode_BGTRU(void);
void vax_Opcode_BLBC(void);
void vax_Opcode_BLBS(void);
void vax_Opcode_BLEQ(void);
void vax_Opcode_BLEQU(void);
void vax_Opcode_BLSS(void);
void vax_Opcode_BNEQ(void);
void vax_Opcode_BRB(void);
void vax_Opcode_BRW(void);
void vax_Opcode_BSBB(void);
void vax_Opcode_BSBW(void);
void vax_Opcode_BVC(void);
void vax_Opcode_BVS(void);
void vax_Opcode_CASEB(void);
void vax_Opcode_CASEW(void);
void vax_Opcode_CASEL(void);
void vax_Opcode_JMP(void);
void vax_Opcode_JSB(void);
void vax_Opcode_RSB(void);
void vax_Opcode_SOBGEQ(void);
void vax_Opcode_SOBGTR(void);

// cpu_call.c
void vax_Call(int32 *, int);
void vax_Opcode_CALLG(void);
void vax_Opcode_CALLS(void);
void vax_Opcode_RET(void);

// cpu_compare.c
void vax_Opcode_CMPB(void);
void vax_Opcode_CMPW(void);
void vax_Opcode_CMPL(void);
void vax_Opcode_TSTB(void);
void vax_Opcode_TSTW(void);
void vax_Opcode_TSTL(void);

// cpu_convert.c
void vax_Opcode_CVTBW(void);
void vax_Opcode_CVTBL(void);
void vax_Opcode_CVTWB(void);
void vax_Opcode_CVTWL(void);
void vax_Opcode_CVTLB(void);
void vax_Opcode_CVTLW(void);

// cpu_field.c
int32 vax_GetField(int32 *, int32);
void vax_Opcode_CMPV(void);
void vax_Opcode_CMPZV(void);
void vax_Opcode_EXTV(void);
void vax_Opcode_EXTZV(void);
void vax_Opcode_FFC(void);
void vax_Opcode_FFS(void);
void vax_Opcode_INSV(void);

// cpu_fpa.c
void vax_Opcode_ACBF(void);
void vax_Opcode_ACBD(void);
void vax_Opcode_ACBG(void);
void vax_Opcode_ADDF(void);
void vax_Opcode_ADDD(void);
void vax_Opcode_ADDG(void);
void vax_Opcode_CMPF(void);
void vax_Opcode_CMPD(void);
void vax_Opcode_CMPG(void);
void vax_Opcode_CVTBD(void);
void vax_Opcode_CVTBF(void);
void vax_Opcode_CVTBG(void);
void vax_Opcode_CVTDB(void);
void vax_Opcode_CVTDF(void);
void vax_Opcode_CVTDL(void);
void vax_Opcode_CVTDW(void);
void vax_Opcode_CVTFB(void);
void vax_Opcode_CVTFD(void);
void vax_Opcode_CVTFG(void);
void vax_Opcode_CVTFL(void);
void vax_Opcode_CVTFW(void);
void vax_Opcode_CVTGB(void);
void vax_Opcode_CVTGF(void);
void vax_Opcode_CVTGL(void);
void vax_Opcode_CVTGW(void);
void vax_Opcode_CVTLD(void);
void vax_Opcode_CVTLF(void);
void vax_Opcode_CVTLG(void);
void vax_Opcode_CVTRDL(void);
void vax_Opcode_CVTRFL(void);
void vax_Opcode_CVTRGL(void);
void vax_Opcode_CVTWD(void);
void vax_Opcode_CVTWF(void);
void vax_Opcode_CVTWG(void);
void vax_Opcode_DIVF(void);
void vax_Opcode_DIVD(void);
void vax_Opcode_DIVG(void);
void vax_Opcode_EMODF(void);
void vax_Opcode_EMODD(void);
void vax_Opcode_EMODG(void);
void vax_Opcode_MNEGF(void);
void vax_Opcode_MNEGD(void);
void vax_Opcode_MNEGG(void);
void vax_Opcode_MOVF(void);
void vax_Opcode_MOVD(void);
void vax_Opcode_MOVG(void);
void vax_Opcode_MULF(void);
void vax_Opcode_MULD(void);
void vax_Opcode_MULG(void);
void vax_Opcode_POLYF(void);
void vax_Opcode_POLYD(void);
void vax_Opcode_POLYG(void);
void vax_Opcode_SUBF(void);
void vax_Opcode_SUBD(void);
void vax_Opcode_SUBG(void);
void vax_Opcode_TSTF(void);
void vax_Opcode_TSTD(void);
void vax_Opcode_TSTG(void);

// cpu_integer.c
void vax_Opcode_ADAWI(void);
void vax_Opcode_ADDB(void);
void vax_Opcode_ADDW(void);
void vax_Opcode_ADDL(void);
void vax_Opcode_ADWC(void);
void vax_Opcode_ASHL(void);
void vax_Opcode_ASHQ(void);
void vax_Opcode_BICB(void);
void vax_Opcode_BICW(void);
void vax_Opcode_BICL(void);
void vax_Opcode_BISB(void);
void vax_Opcode_BISW(void);
void vax_Opcode_BISL(void);
void vax_Opcode_BITB(void);
void vax_Opcode_BITW(void);
void vax_Opcode_BITL(void);
void vax_Opcode_CLRB(void);
void vax_Opcode_CLRW(void);
void vax_Opcode_CLRL(void);
void vax_Opcode_CLRQ(void);
void vax_Opcode_DECB(void);
void vax_Opcode_DECW(void);
void vax_Opcode_DECL(void);
void vax_Opcode_DIVB(void);
void vax_Opcode_DIVW(void);
void vax_Opcode_DIVL(void);
void vax_Opcode_EDIV(void);
void vax_Opcode_EMUL(void);
void vax_Opcode_INCB(void);
void vax_Opcode_INCW(void);
void vax_Opcode_INCL(void);
void vax_Opcode_MULB(void);
void vax_Opcode_MULW(void);
void vax_Opcode_MULL(void);
void vax_Opcode_PUSHL(void);
void vax_Opcode_ROTL(void);
void vax_Opcode_SBWC(void);
void vax_Opcode_SUBB(void);
void vax_Opcode_SUBW(void);
void vax_Opcode_SUBL(void);
void vax_Opcode_XORB(void);
void vax_Opcode_XORW(void);
void vax_Opcode_XORL(void);

// cpu_intexc.c
int32 vax_EvaluateIRQ(void);
int32 vax_GetVector(void);
void  vax_Exception(int32, int32, int32, ...);
void  vax_Emulate(int32);
void  vax_Opcode_CHMK(void);
void  vax_Opcode_CHME(void);
void  vax_Opcode_CHMS(void);
void  vax_Opcode_CHMU(void);
void  vax_Opcode_LDPCTX(void);
void  vax_Opcode_SVPCTX(void);
void  vax_Opcode_REI(void);

// cpu_main.c
char *vax_DisplayCondition(void);
//void vax_DecodeOperand(INSTRUCTION *, int32 *);
void vax_DecodeOperand(uint32 *, int32 *);
int  vax_Execute(MAP_DEVICE *);

// cpu_misc.c - Miscellaneous Instructions
void vax_Opcode_BICPSW(void);
void vax_Opcode_BISPSW(void);
void vax_Opcode_BPT(void);
void vax_Opcode_BUGL(void);
void vax_Opcode_BUGW(void);
void vax_Opcode_HALT(void);
void vax_Opcode_INDEX(void);
void vax_Opcode_MOVPSL(void);
void vax_Opcode_NOP(void);
void vax_Opcode_POPR(void);
void vax_Opcode_PUSHR(void);
void vax_Opcode_XFC(void);
void vax_Opcode_Illegal(void);
void vax_Opcode_Unimplemented(void);
void vax_Opcode_Emulate(void);

// cpu_mm.c
TLBENT  vax_Fill(uint32, int32, int32, int32 *);
void    vax_ClearTBTable(boolean);
void    vax_ClearTBEntry(uint32);
boolean vax_CheckTBEntry(uint32);
void    vax_Opcode_PROBER(void);
void    vax_Opcode_PROBEW(void);
int     vax_ShowTLB(void *, int, char **);

// cpu_move.c
void vax_Opcode_MCOMB(void);
void vax_Opcode_MCOMW(void);
void vax_Opcode_MCOML(void);
void vax_Opcode_MNEGB(void);
void vax_Opcode_MNEGW(void);
void vax_Opcode_MNEGL(void);
void vax_Opcode_MOVB(void);
void vax_Opcode_MOVW(void);
void vax_Opcode_MOVL(void);
void vax_Opcode_MOVQ(void);
void vax_Opcode_MOVZBW(void);
void vax_Opcode_MOVZBL(void);
void vax_Opcode_MOVZWL(void);

// cpu_queue.c
void vax_Opcode_INSQHI(void);
void vax_Opcode_INSQTI(void);
void vax_Opcode_INSQUE(void);
void vax_Opcode_REMQHI(void);
void vax_Opcode_REMQTI(void);
void vax_Opcode_REMQUE(void);

// cpu_string.c
void vax_Opcode_CMPC3(void);
void vax_Opcode_CMPC5(void);
void vax_Opcode_LOCC(void);
void vax_Opcode_MOVC3(void);
void vax_Opcode_MOVC5(void);
void vax_Opcode_SCANC(void);
void vax_Opcode_SKPC(void);
void vax_Opcode_SPANC(void);

// cpu_system.c
void vax_Opcode_MFPR(void);
void vax_Opcode_MTPR(void);

// dev_cty.c
VAX_CONSOLE *vax_ConsoleInit(VAX_CPU *);
void vax_ConsoleCleanup(VAX_CONSOLE *);
uint32 vax_ReadRXCS(VAX_CONSOLE *);
uint32 vax_ReadRXDB(VAX_CONSOLE *);
uint32 vax_ReadTXCS(VAX_CONSOLE *);
uint32 vax_ReadTXDB(VAX_CONSOLE *);
void   vax_WriteRXCS(VAX_CONSOLE *, uint32);
void   vax_WriteRXDB(VAX_CONSOLE *, uint32);
void   vax_WriteTXCS(VAX_CONSOLE *, uint32);
void   vax_WriteTXDB(VAX_CONSOLE *, uint32);

// memory.c
int    vax_InitMemory(VAX_CPU *, int);
int    vax_FreeMemory(VAX_CPU *);
int32  vax_ReadInst(int32);
uint32 vax_ReadAligned(uint32, int32);
uint32 vax_ReadUnaligned(uint32, int32);
uint32 vax_Read(uint32, int32, int32);
void   vax_WriteAligned(uint32, uint32, int32);
void   vax_WriteUnaligned(uint32, uint32, int32);
void   vax_Write(uint32, uint32, int32, int32);
int32  vax_Test(uint32, int32, int32 *);

// system.c
void *vax_Create(MAP_DEVICE *, MAP_DEVICE *, int, char **);
int  vax_LoadBoot(VAX_SYSTEM *, char *, uint32);
int  vax_LoadROM(VAX_SYSTEM *, char *, uint32);
void vax_CheckInstructions(void);

// ka780.c (KA780 System Configurations)
int32 ka780_prRead(int32);
void  ka780_prWrite(int32, int32);
void  ka780_Initialize(VAX_CPU *);
